# check_config.py -- Validate P4DTI configuration parameters.
#
# Nick Barnes, Ravenbrook Limited, 2001-01-18
#
# $Id: //info.ravenbrook.com/project/p4dti/version/1.0/code/replicator/check_config.py#3 $
# 
# Copyright 2000 Ravenbrook Limited.  This document is provided "as is",
# without any express or implied warranty. In no event will the authors
# be held liable for any damages arising from the use of this document.
# You may make and distribute copies and derivative works of this
# document provided that (1) you do not charge a fee for this document or
# for its distribution, and (2) you retain as they appear all copyright
# and licence notices and document history entries, and (3) you append
# descriptions of your modifications to the document history.

import re
import types

error = "P4DTI configuration error"

def check_bool(param, name):    
    if (param != 0 and param != 1):
        raise error, ("Configuration parameter '%s' must be 0 or 1." %
                      name)

def check_date(param, name):
    check_string(param, name)
    date_re = "^\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d$"
    if not re.match(date_re, param):
        raise error, ("Configuration parameter '%s' (value '%s') is not a "
                      "valid date.  The right format is 'YYYY-MM-DD HH:MM:SS'."
                      % (name, param))        

def check_email(param, name):
    check_string(param, name)

    # RFC 822 defines an 'addr-spec' as follows.
    #
    #   addr-spec   =  local-part "@" domain
    #   local-part  =  word *("." word)
    #   word        =  atom
    #   domain      =  sub-domain *("." sub-domain)
    #   sub-domain  =  domain-ref
    #   domain-ref  =  atom
    #   atom        =  1*<any CHAR except specials, SPACE and CTLs>
    #   specials    =  "(" / ")" / "<" / ">" / "@" /  "," / ";" / ":" / "\"
    #   		   / <"> /  "." / "[" / "]"
    #
    # (A CHAR is an ASCII character.)
    #
    # This isn't the full story.  "Structured field bodies" in RFC822 also
    # permit quoting, domain literals and comments.  This code doesn't
    # recognise these features.  So users of the P4DTI will have to get by
    # without.
    atom_re = "[!#$%&'*+\\-/0-9=?A-Z^_`a-z{|}~]+"
    email_re = "^%s(\\.%s)*@%s(\\.%s)*$" % (atom_re, atom_re, atom_re, atom_re)

    if not re.match(email_re, param):
        raise error, ("Configuration parameter '%s' (value '%s') is not a "
                      "valid e-mail address." % (name, param))

def check_function(param, name):
    if not isinstance(param, types.FunctionType):
        raise error, ("Configuration parameter '%s' must be a function." %
                      name)

def check_host(param, name):
    check_string(param, name) # could do more, e.g. look it up.

def check_int(param, name):    
    if not isinstance(param, types.IntType):
        raise error, ("Configuration parameter '%s' must be an integer." %
                      name)

def check_list_of(param, name, type, typename):
    if not isinstance(param, types.ListType):
        raise error, ("Configuration parameter '%s' must be a list." % name)
    for item in param:
        if not isinstance(item, type):
            raise error ("Configuration parameter '%s' must be a list of %s." %
                         (name, typename))

def check_string(param, name):
    if not isinstance(param, types.StringType):
        raise error, ("Configuration parameter '%s' must be a string." %
                      name)

def check_string_or_none(param, name):
    if not (param == None or
            isinstance(param, types.StringType)):
        raise error, ("Configuration parameter '%s' must be None or a string."
                      % name)

# A. REFERENCES
#
# [RFC 822] "RFC 822: Standard for the format of ARPA Internet text messages";
# David H Crocker; 1982-08-13; <ftp://src.doc.ic.ac.uk/rfc/rfc822.txt>.
#
#
# B. DOCUMENT HISTORY
#
# 2001-01-18 NB Moved from configure_bugzilla.py so we can share with other
# DTs.
#
# 2001-01-23 GDR Extended check_email so that it checks RFC822 address syntax.
#
# 2001-02-04 GDR Alphabetized.  Added check_date.
#
# 2001-02-16 NB Added check_function (for checking replicate_p parameter).
